@@ -31,3 +31,6 @@ from .nparray2xmf import *
 from .vector_actions import *
 from .showy import *
 from .sample_curve_from_cloud import *
+from .same_nob import *
+
+from .axipointcloud import *
