@@ -1,7 +1,6 @@
 """ module to data array-like dictionnary to files
     for visulaisation or storage pupopses
 """
-import imp
 import os
 import numpy as np
 from arnica.utils.nparray2xmf import (create_time_collection_xmf,
@@ -42,19 +41,20 @@ def dump_dico_2d_nparrays(data_dict, filename,
 
     fields = list(data_dict.keys())
 
-
     visu_ = NpArray2Xmf(filename,
                         time=time,
                         mesh_name=grid_name,
                         domain_name=domain_name)
     visu_.create_grid(x_coords, y_coords, z_coords)
     for field in fields:
-        visu_.add_field(data_dict[field], field)
+        if field != 'xyz':
+            visu_.add_field(data_dict[field], field)
     visu_.dump()
 
 def dump_dico_2d_time_nparrays(data_dict, root_path, prefix,
                                x_coords, y_coords, z_coords,
                                **kw_args):
+    #pylint: disable=too-many-arguments, too-many-locals
     """ Dumps a dictionnary of time series 2d arrays
        to xmf files
 
@@ -97,9 +97,9 @@ def dump_dico_2d_time_nparrays(data_dict, root_path, prefix,
     steps = kw_args.get('steps')
 
     if times is None:
-        times = np.linspace(0.0, n_time, num=n_time, dtype=float)
-    if "steps" is None:
-        steps = np.linspace(0, n_time, num=n_time, dtype=int)
+        times = np.linspace(0.0, n_time - 1., num=n_time, dtype=float)
+    if steps is None:
+        steps = np.linspace(0, n_time - 1, num=n_time, dtype=int)
 
     collection = []
     for step in range(n_time):
@@ -119,6 +119,7 @@ def dump_dico_2d_time_nparrays(data_dict, root_path, prefix,
 
 
 def dump_dico_1d_nparrays(filename, data_dict):
+    #pylint: disable=too-many-locals
     """Write statistics to file
 
         Parameters:
@@ -133,30 +134,31 @@ def dump_dico_1d_nparrays(filename, data_dict):
         =======
         None
     """
-    try:
-        imp.find_module('pandas')
-        import pandas as pd
-        pd_found = True
-    except ImportError:
-        msg = 'dump_dico_1d_nparrays cannot find pandas, the classic csv\
-               writing will be used. Pandas is required to dump results to\
-               Excel files'
-        print(msg)
-        pd_found = False
 
     file_out = os.path.abspath(filename)
     if not os.path.exists(os.path.dirname(file_out)):
         os.makedirs(os.path.dirname(file_out))
-
-
     file_out, format_ = os.path.splitext(os.path.abspath(file_out))
     format_ = format_.replace('.', '').lower()
 
-    if format_ not in ["csv", "xlsx"]:
-        format_ = 'xlsx'
+    if format_ not in ['csv', 'xlsx']:
+        msg = ("Extension", format_, "is not supported. Supported extensions : "
+               "'csv', 'xlsx'. Default extension 'csv' will be used.")
+        print("WARNING [dump_dico_1d_nparrays] : ", msg)
+        format_ = 'csv'
 
+    elif format_ == "xlsx":
+        try:
+            #pylint: disable=import-outside-toplevel, import-error, unused-import
+            import pandas as pd
+            import openpyxl
+        except ImportError as import_error:
+            msg = ("dump_dico_1d_nparrays cannot find 'pandas' neither/nor "
+                   "'openpyxl' packages. Import those packages or use '.csv' "
+                   "extension to dump classic csv files instead.")
+            raise ImportError(msg) from import_error
 
-    if format_ == "xlsx" and pd_found:
+    if format_ == 'xlsx':
         file_out = "%s.%s" %(file_out, format_)
         df_ = pd.DataFrame.from_dict(data_dict)
         df_.to_excel(file_out, index=False)
@@ -171,7 +173,6 @@ def dump_dico_1d_nparrays(filename, data_dict):
             out_array[:, col] = data_dict[key][:]
         np.savetxt(file_out, out_array, delimiter=' , ', header=header)
 
-
 def dump_dict2xmdf(filename, grid, data_dict):
     """
     *Dump 2D matrices into hdf5 file*
@@ -181,6 +182,9 @@ def dump_dict2xmdf(filename, grid, data_dict):
     :param grid: Array of xyz-coordinates of shape (n_v, n_u, 3)
     :param data_dict: Dict of field arrays of shape (n_v, n_u)
     """
+
+    print("DEPRECATED TOOL. USE dump_dico_2d_nparrays() INSTEAD.")
+
     if '.h5' not in filename:
         filename += '.h5'
 
@@ -211,44 +215,41 @@ def dump_dico_0d(filename, data_dict):
         None
     """
 
-    try:
-        imp.find_module('pandas')
-        import pandas as pd
-        pd_found = True
-    except ImportError:
-        msg = 'dump_dico_1d_nparrays cannot find pandas, the classic csv\
-               writing will be used. Pandas is required to dump results to\
-               Excel files'
-        print(msg)
-        pd_found = False
-
     file_out = os.path.abspath(filename)
     if not os.path.exists(os.path.dirname(file_out)):
         os.makedirs(os.path.dirname(file_out))
     file_out, format_ = os.path.splitext(os.path.abspath(file_out))
     format_ = format_.replace('.', '').lower()
 
-    if format_ not in ["csv", "xlsx"]:
-        format_ = 'xlsx'
+    if format_ not in ['csv', 'xlsx']:
+        msg = ("Extension", format_, "is not supported. Supported extensions : "
+               "'csv', 'xlsx'. Default extension 'csv' will be used.")
+        print("WARNING [dump_dico_0d] : ", msg)
 
-    if format_ == "xlsx" and pd_found:
+    elif format_ == "xlsx":
+        try:
+            #pylint: disable=import-outside-toplevel, import-error, unused-import
+            import pandas as pd
+            import openpyxl
+        except ImportError as import_error:
+            msg = ("dump_dico_1d_nparrays cannot find 'pandas' neither/nor "
+                   "'openpyxl' packages. Import those packages or use '.csv' "
+                   "extension to dump classic csv files instead.")
+            raise ImportError(msg) from import_error
+
+    if format_ == 'xlsx':
         file_out = "%s.%s" %(file_out, format_)
         df_ = pd.DataFrame(data_dict, index=[0])
         df_.to_excel(file_out, index=False)
-
     else:
         format_ = 'csv'
         file_out = "%s.%s" %(file_out, format_)
         header = " , ".join(list(data_dict.keys()))
         n_columns = len(data_dict.keys())
-        n_lines = data_dict[next(iter(data_dict.keys()))].shape[0]
-        out_array = np.zeros((n_lines, n_columns))
+        out_array = np.zeros(n_columns)
         for col, key in enumerate(data_dict, 0):
-            out_array[:, col] = data_dict[key][:]
-        np.savetxt(file_out, out_array, delimiter=' , ', header=header)
-
-
-
+            out_array[col] = data_dict[key]
+        np.savetxt(file_out, out_array[np.newaxis, :], delimiter=' , ', header=header)
 
 def plot_dict_data_as_file(data_dict, filename, x_data, y_data, **kw_args):
     """ Generates and write XY-plot to file
@@ -268,6 +269,7 @@ def plot_dict_data_as_file(data_dict, filename, x_data, y_data, **kw_args):
     x_label : label of the x axis, by default x_data is used (supports latex)
     y_label : label of the y axis, by default y_data is used (supports latex)
     """
+    #pylint: disable=import-outside-toplevel
     import matplotlib.pyplot as plt
 
     x_label = kw_args.get('x_label')
@@ -288,7 +290,7 @@ def plot_dict_data_as_file(data_dict, filename, x_data, y_data, **kw_args):
         format_ = 'pdf'
 
 
-    plt.rc('text', usetex=True)
+    #plt.rc('text', usetex=True)
     plt.rc('font', family='serif')
     fig = plt.figure(figsize=(5.5, 5))
     fig.subplots_adjust(left=0.15, right=0.97, bottom=0.15,
