@@ -13,8 +13,9 @@ OST : Seven nation Army (Westworld), R. Djawadi
 """
 
 from collections import namedtuple
-from scipy import spatial
+import warnings
 import numpy as np
+from scipy.spatial.ckdtree import cKDTree
 
 BIG = 1e6
 
@@ -26,12 +27,16 @@ def projection_kdtree(points,
     """
     Project the n points following the direction on the m suface.
 
-    :param points: "p" nparray of shape (n,3),
-    :param directions: nparray of shape (n,3),
-    :param point_surface: "s" nparray of shape (m,3),
-    :param normal_surface: nparray of shape (m,3),
-    :param neigbors: nb of neigbors to take into account
-    :param tol: maximum distance beyond cyl dist witl bi set to BIG
+    :param points: Array of [p] points of shape (p,3)
+    :param directions: Array of [p] direction vectors of shape (p,3)
+    :param point_surface: Array of [n] surface nodes of shape (n,3)
+    :param normal_surface: Array of [n] surface node normals of shape (n,3)
+    :param neigbors: Number [k] of neigbors to take into account
+    :type neigbors: int
+    :param tol: Maximum distance beyond cyl dist with big set to BIG
+    :type tol: float
+    :param project: If True, first project points along normal.
+    :type project: bool
 
     :returns:
         - **projected_pts** - "t" nparray of shape (n,3), projected on the surface
@@ -52,17 +57,19 @@ def projection_kdtree(points,
 
               align : alignment (pscal of two unit vectors, in [-1,1])
 
-    - compute a kdtree, and reduce the surfacepoints to a neighborhood indexes [n,k]
-    - iterate over all points [n]:
-    - compute the distance surface_dist to the surface, for each neighbor [k]
-    - compute the alignment align, scal. product of normal and direction [k]
-    - compute all points all_pts moved on the surface [k,3]
-    - compute shperical distance  for each point to its moved point [k]
-    - get the closest point idx closest_pt
-    - recompute the cylindrical distance cyl_dist for all the neigborhood
+    Algorithm :
+        - If project bool is True, first compute [p] projection from [p] points along the [p,1] \
+          spherical closest node's normal. If False, projected_points = points.
+        - Reduce computation to the [p,k] spherical closest nodes of the [p] projected_points
+        - Compute the [p,k] cylindrical distances from the [p,k] closest nodes \
+          to the [p] lines defined by the [p] projected points and the [p] direction vectors.
     """
-
-    opts = {'neigbors': 1000, 'tol': 1000.0}
+    opts = {'neigbors': 1000, 'tol': 1000.0, 'project': True}
+    if 'proj_axis' in kwargs:
+        warnings.warn("'proj_axis' kwarg is deprecated. Please use boolean 'project' "
+                      "kwarg instead : project = True if proj_axis = 'nml', false instead.")
+        if kwargs['proj_axis'] == 'dir':
+            opts['project'] = False
     for keyword in opts:
         if keyword in kwargs:
             opts[keyword] = kwargs[keyword]
@@ -76,30 +83,33 @@ def projection_kdtree(points,
           + str(opts['neigbors'])
           )
 
-    # First step : find the closest point to the drill from the skin_data points
-    kdtree = spatial.cKDTree(point_surface) # pylint: disable=not-callable
-    _, index = kdtree.query(points, k=1)
+    kdtree = cKDTree(point_surface) # pylint: disable=not-callable
+    if opts['project']:
+        _, index = kdtree.query(points, k=1)
+        projected_pts = project_points(
+            points,
+            normal_surface[index],
+            point_surface[index][:, np.newaxis, :]).proj_pts
+        projected_pts = np.squeeze(projected_pts, axis=1)
+    else:
+        projected_pts = points
 
-    # Second step : project drill points on the surface along normal's closest point
-    projected_pts = project_points(points,
-                                   normal_surface[index],
-                                   point_surface[index][:, np.newaxis, :]).proj_pts
-    projected_pts = np.squeeze(projected_pts, axis=1)
-
-    # Third step : compute cylindrical distances of the k-neigbor points.
     _, indexes = kdtree.query(projected_pts, k=opts["neigbors"])
-    cyl_dists = compute_cyl_dist(projected_pts,
-                                 directions,
-                                 point_surface[indexes],
-                                 normal_surface[indexes],
-                                 opts["tol"])
+    axi_dists, cyl_dists = compute_dists(
+        projected_pts,
+        directions,
+        point_surface[indexes],
+        normal_surface[indexes],
+        opts["tol"])
 
-    Proj_KDTree = namedtuple("Proj_KDTree", ["moved_pts", "indexes", "cyl_dists"])
-
-    return Proj_KDTree(projected_pts, indexes, cyl_dists)
+    Proj_KDTree = namedtuple(
+        "Proj_KDTree",
+        ("moved_pts", "indexes", "axi_dists", "cyl_dists")
+    )
+    return Proj_KDTree(projected_pts, indexes, axi_dists, cyl_dists)
 
 
-def compute_cyl_dist(points, directions, points_surf, normals_surf, tol):
+def compute_dists(points, directions, points_surf, normals_surf, tol):
     """
     *Compute cylindrical distances*
 
@@ -127,7 +137,7 @@ def compute_cyl_dist(points, directions, points_surf, normals_surf, tol):
     proj = project_points(points, directions, points_surf)
     cyl_dists = proj.rad_dists
     axi_dists = proj.axi_dists
-    align = np.sum(np.multiply(normals_surf, directions[:, np.newaxis, :]), axis=-1)
+    align = np.sum(np.multiply(normals_surf, directions[:, None, :]), axis=-1)
     cyl_dists = np.where(align < 0.0,
                          cyl_dists,
                          BIG*np.ones(cyl_dists.shape))
@@ -135,7 +145,7 @@ def compute_cyl_dist(points, directions, points_surf, normals_surf, tol):
                          cyl_dists,
                          BIG*np.ones(cyl_dists.shape))
 
-    return cyl_dists
+    return axi_dists, cyl_dists
 
 def project_points(points_source, normals, points_target):
     r"""\
@@ -199,10 +209,48 @@ def project_points(points_source, normals, points_target):
     axi_dists = np.sum(np.multiply((points_target - points_source),
                                    normals),
                        axis=-1)
-    projected_points = points_source + np.multiply(normals, axi_dists[:, :, np.newaxis])
+    projected_points = points_source + \
+        np.multiply(normals, axi_dists[:, :, np.newaxis])
 
     rad_dists = np.linalg.norm(points_target - projected_points, axis=-1)
 
     Proj = namedtuple('Proj', ['proj_pts', 'axi_dists', 'rad_dists'])
 
     return Proj(projected_points, axi_dists, rad_dists)
+
+
+def intersect_plan_line(xyz_line, vec_line, xyz_plan, nml_plan):
+    r"""\
+    *Compute intersection coordinates of a line and a plan*
+
+        - Line defined by a point `xyz_line` and a vector `vec_line`
+        - Plan defined by a point `xyz_plan` and a normal `nml_plan`
+
+    Arrays dimensions must be consistent together.
+
+    ::
+        nml_plan   xyz_line
+           A         x
+           |        /
+           |       /  vec_line
+        ___x______I________
+        xyz_plan   \
+                    Intersection point
+
+    :param xyz_line: Array of coordinates of shape (3,) or (n, 3)
+    :param vec_line: Array of components of shape (3,) or (n,3)
+    :param xyz_plan: Array of coordinates of shape (3,) or (n,3)
+    :param nml_plan: Array of complonents of shape (3,) or (n,3)
+
+    :returns: **xyz_intersect** - Array of coordinates of shape (3,) or (n,3)
+    """
+    vec_dot_nml = np.sum(np.multiply(vec_line, nml_plan), axis=-1)
+    nml_dot_pos = np.sum(np.multiply(nml_plan, xyz_plan - xyz_line), axis=-1)
+    parameter = nml_dot_pos / vec_dot_nml
+
+    try:
+        xyz_intersect = xyz_line + vec_line * parameter[:, None]
+    except IndexError:
+        xyz_intersect = xyz_line + vec_line * parameter
+
+    return xyz_intersect
