@@ -392,10 +392,11 @@ class Shell():
         # Build crest
         spline_order = min(len(ctrl_pts_1) - 1, MAX_SPLINE_ORDER)
         # Generate continuous spline from control points.
-        tck, _ = interpolate.splprep([ctrl_pts_1,
-                                      ctrl_pts_2],
-                                     s=SPLINE_SMOOTHNESS,
-                                     k=spline_order)
+        tck, _ = interpolate.splprep(
+            [ctrl_pts_1, ctrl_pts_2],
+            s=SPLINE_SMOOTHNESS,
+            k=spline_order,
+        )
         unew = np.linspace(0, 1, num=self.shape[1])
         # Generate discrete spline
         # Numpy Array of dim (2, self.shape[1]) [x,r] / [x,y]
