@@ -15,8 +15,7 @@ SMALL = 1e-12
 
 
 def renormalize(np_ar_vect):
-    """ renormalize a numpy array of vectors,
-    considering the last axis 
+    """ renormalize a numpy array of vectors considering the last axis
     """
     result = (np_ar_vect
               / np.expand_dims(np.linalg.norm(np_ar_vect, axis=-1),
@@ -31,7 +30,7 @@ def angle_btw_vects(np_ar_vect1, np_ar_vect2, convert_to_degree=False):
 
     # Clipping is needed when pscal.max() > 1 : 1.000000000002
     sdot = np.clip(sdot, -1.0, 1.0)
-    angles = np.arcsin(sdot)
+    angles = np.arccos(sdot)
     if convert_to_degree:
         angles = np.rad2deg(angles)
     return angles
@@ -93,6 +92,25 @@ def rotate_vect_around_x(np_ar_vect, angle_deg):
 
     return result
 
+def rotate_vect_around_axis(xyz, *tuples_rot):
+    """
+    *Rotate vector around vector or series of vector*
+
+    :param xyz: Array of xyz-coordinates of shape (n,3)
+    :param tuples_rot: List of tuple with rotation data : \
+                       Axis array of shape (3,) axis, \
+                       Float angle in degree
+
+    :returns: Array of rotated xyz-coordinates of shape (n,3)
+    """
+    quat = Rot.from_rotvec([0, 0, 0])
+    for axis, angle in tuples_rot:
+        rot_vect = np.multiply(renormalize(axis),
+                               angle * np.pi / 180.)
+        quat *= Rot.from_rotvec(rot_vect)
+
+    return quat.apply(xyz)
+
 def dilate_vect_around_x(azimuth, np_ar_vect, angle_deg_init=None, angle_deg_targ=360):
     """ dilate vectors around axis x from a specified initial range angle
         to a target range angle.
@@ -226,18 +244,22 @@ def vect_to_quat(vect_targ, vect_source):
     :type vect_source: np.array
 
     :return:
-    
+
         - **quat** - Array of quaternion of dim (n,)
     """
     vect_targ = renormalize(vect_targ)
     vect_source = renormalize(vect_source)
 
     prod_scal = np.clip(np.sum(vect_targ * vect_source,
-                               axis = -1),
+                               axis=-1),
                         -1, 1)
+
     rot_ang = np.arccos(prod_scal)
     if vect_targ.ndim >= 2:
         rot_ang = rot_ang[:, np.newaxis]
+    else:
+        rot_ang = np.array([rot_ang])
+
     rot_axe = np.cross(vect_source, vect_targ)
     idx = np.where(rot_ang > 1e-6)[0]
     rot_axe[idx] = renormalize(rot_axe[idx])
@@ -250,12 +272,12 @@ def vect_to_quat(vect_targ, vect_source):
 
 
 def make_radial_vect(coord, vects):
-    """ 
+    """
     *Recalibrate vectors to make them radial.*
-    
+
     The vectors are readjusted to cross x-axis. It is mainly done for nodes
     on the limit of the boundary for axi-cylindrical geometries.
-    
+
     :param coord: Array of dim (n,3) of float coordinates
     :type coord: np.array
     :param vects: Array of dim (n,3) of float components
@@ -275,54 +297,6 @@ def make_radial_vect(coord, vects):
     radial_vect = rotate_vect_around_x(vects, diff_angle)
     return radial_vect
 
-def clip_by_bounds(points_coord, bounds_dict, keep="in", returns=True):
-    """
-    *Clip a cloud by keeping only or removing a bounded region*
-
-    The dict to provide must be filled as follow :
-    bounds_dict = {component_1 : (1_min, 1_max),
-                   compoment_2 : (2_min, 2_max),
-                   ...}
-
-    component_1 = ["x", "y", "z", "r", "theta"]
-
-    The bounded region can either be :
-
-        - A 1D slice if only 1 component is provided ;
-        - A 2D box if 2 components are provided ;
-        - A 3D box if 3 components are provided.
-
-    If keep="in", returns the point coordinates inside the bounds.
-    If keep="out", returns the point coordinates outside the bounds.
-
-    If returns=True, returns the coordinates clipped
-    If returns=False, returns the mask of boolean than can be applied\
-    on other arrays
-
-    :param point_cloud: Array of dim (n,k) of coordinates
-    :type point_cloud: np.array
-    :param bounds_dict: Dict of MAX lengh k of tuple of floats
-    :param keep: Either keeps what is inside or outside
-    :type keep: str
-
-    :returns:
-
-        - **points_coord_clipped** - Array of dim (m,k) with m<=n
-        OR
-        - **mask** - Array of dim (n,) of booleans
-    """
-    mask = np.ones(points_coord.shape[0], dtype=bool)
-    for dim, bounds in bounds_dict.items():
-        mask &= mask_cloud(points_coord, dim, bounds)
-
-    if keep == "out":
-        mask = np.invert(mask)
-
-    if returns:
-        return points_coord[mask]
-    else:
-        return mask
-
 def mask_cloud(np_ar_xyz,
                axis,
                support):
